#include <midasplayer.h>
#include <cstring.h>
#include <classlib\arrays.h>
#include <stdlib.h>
#include <midasdll.h>
#include <fstream.h>

void lpDMM::RetrieveSongInfo()
 {
  MIDASmodule hModule = MIDASloadModule((char*) fn.c_str());
  if (hModule)
   {
    MIDASmoduleInfo SongInfo;
    MIDASgetModuleInfo(hModule, &SongInfo);
    OemToChar(SongInfo.songName, SongInfo.songName);
    MIDASfreeModule(hModule);
    SetTrackName(SongInfo.songName);
   }
  else
   SetTrackName(NO_TRACK_INFO);
 }

lpMIDASPlayer::lpMIDASPlayer()
 : Modules(10, 0, 4), PlayOrder(SequentialForward), Clipping(Clip), State(Nothing),
  hModule(0), hPlay(0), TrackChange(0)
 {
  ActiveDMM = 0;
  randomize();
  midas_Init();
 }

lpMIDASPlayer::~lpMIDASPlayer()
 {
  Unload();
  midas_Done();
 }

void lpMIDASPlayer::SetVolume(int vol)
 {
  iVolume = vol;
  if (GetState(Playing))
   MIDASsetMusicVolume(hPlay, vol);
 }

void lpMIDASPlayer::midas_Init()
 {
  MIDASstartup();
  MIDASinit();
  MIDASstartBackgroundPlay(50);
 }

void lpMIDASPlayer::midas_Done()
 {
  MIDASclose();
 }

void lpMIDASPlayer::FreePlayList()
 {
  Modules.Flush();
 }

void lpMIDASPlayer::ReadPlayList(const string& fn)
 {
  if (fn != NO_FILE_SPEC)
   ListFN = fn;

  fstream fin(ListFN.c_str(), ios::in | ios::nocreate);
  if (fin.fail()) return;

  while (!fin.eof())
   {
    string ln;
    getline(fin, ln);
    int p = ln.find("','");
    if (p!= NPOS)
     {
      string modfn(ln, 1, p-1);
      string modtrack(ln, p+3, ln.length()-(p+3)-1);
      AddModule(modfn, modtrack);
     }
   }
 }

void lpMIDASPlayer::WritePlayList()
 {
  fstream fout(ListFN.c_str(),  ios::out | ios::trunc);
  if (fout.fail()) return;

  lpDMMListIterator i(Modules);
  while (i)
   {
    lpDMM *dmm = i.Current();
    string ln = "'"+dmm->GetFileName()+"','"+dmm->GetTrackName()+"'\r\n";
    fout << ln;
    i++;
   };
 }

void lpMIDASPlayer::SetPlayListName(const string&fn)
 {
  ListFN = fn;
 }

string lpMIDASPlayer::GetPlayListName()
 {
  return ListFN;
 }

lpDMM* lpMIDASPlayer::AddModule(const string& fn, const string& track)
 {
  if (FindFileDMM(fn) == 0)
   {
    lpDMM* dmm = new lpDMM(fn, track);
    Modules.Add(dmm);
    return dmm;
   }
  return 0;
 }

void lpMIDASPlayer::DeleteIndex(int idx)
 {
  DeleteDMM(FindIndexDMM(idx));
 }

void lpMIDASPlayer::DeleteTrack(const string &track)
 {
  DeleteDMM(FindTrackDMM(track));
 }

void lpMIDASPlayer::DeleteFile(const string &fn)
 {
  DeleteDMM(FindFileDMM(fn));
 }

void lpMIDASPlayer::DeleteDMM(lpDMM *dmm)
 {
  if (dmm)
   {
    if (dmm == ActiveDMM) Unload();
    Modules.Destroy(dmm);
   }
 }

void lpMIDASPlayer::Load()
 {
  if (GetState(Loaded)) Unload();

  if (ActiveDMM)
   {
    hModule = MIDASloadModule((char*) ActiveDMM->GetFileName().c_str());
    if (hModule)
     {
      SetState(1, Loaded);

      MIDASgetModuleInfo(hModule, &SongInfo);

      OemToChar(SongInfo.songName, SongInfo.songName);
      ActiveDMM->SetTrackName(SongInfo.songName);
     }
   }

  TrackChange = true;
 }

void lpMIDASPlayer::Unload()
 {
  if (GetState(Loaded))
   {
    if (GetState(Playing)) Stop();
    MIDASfreeModule(hModule);
    hModule = 0;
   }

  SetState(0, Loaded);
 }

void lpMIDASPlayer::PlayIndex(int idx)
 {
  ActiveDMM = FindIndexDMM(idx);
  Load();
  Play();
 }

void lpMIDASPlayer::PlayFile(const string &fn)
 {
  ActiveDMM = FindFileDMM(fn);
  Load();
  Play();
 }

void lpMIDASPlayer::PlayTrack(const string &track)
 {
  ActiveDMM = FindTrackDMM(track);
  Load();
  Play();
 }

void lpMIDASPlayer::PlayDMM(lpDMM *dmm)
 {
  ActiveDMM = dmm;
  Load();
  Play();
 }

void lpMIDASPlayer::Play(int pos)
 {
  if (GetState(Playing)) return;
  if (!GetState(Loaded)) Load();

  if (GetState(Loaded))
   {
    hPlay = MIDASplayModuleSection(hModule, pos, SongInfo.songLength-1, 0, 1);
    if (hPlay)
     SetState(1, Playing);

    SetVolume();
    UpdateSongInfo();
   }

  if (GetState(Paused))
   Resume();
 }

void lpMIDASPlayer::Stop()
 {
  if (GetState(Playing))
   MIDASstopModule(hPlay);

  if (GetState(Paused)) Resume();

  SetState(0, Playing);
 }

void lpMIDASPlayer::Pause()
 {
  if (!GetState(Paused) && GetState(Playing))
   MIDASsuspend();
  SetState(1, Paused);
 }

void lpMIDASPlayer::Resume()
 {
  if (GetState(Paused))
   MIDASresume();
  SetState(0, Paused);
 }

lpDMM* lpMIDASPlayer::FindIndexDMM(int idx)
 {
  if ((idx < 0) || (idx > Modules.GetItemsInContainer() -1))
   return 0;
  else
   return Modules[idx];
 }

lpDMM* lpMIDASPlayer::FindTrackDMM(const string& track)
 {
  lpDMMListIterator i(Modules);
  while (i)
   {
    if (i.Current()->GetTrackName() == track)
     return i.Current();
    i++;
   }
  return 0;
 }

lpDMM* lpMIDASPlayer::FindFileDMM(const string& fn)
 {
  lpDMMListIterator i(Modules);
  while (i)
   {
    if (i.Current()->GetFileName() == fn)
     return i.Current();
    i++;
   }
  return 0;
 }

void lpMIDASPlayer::PlayNextModule(lpPlayOrder po, lpClipping clp)
 {
  int idx;
  bool play = 1;

  if (Modules.GetItemsInContainer() >= 1)
   {
    lpDMM *olddmm = ActiveDMM;

    if (ActiveDMM != 0)
     idx = Modules.Find(ActiveDMM);
    else
     idx = INT_MAX;

    switch (po)
     {
      case SequentialForward :
       if (idx != INT_MAX) idx++; else idx = 0; break;
      case SequentialBackward :
       if (idx != INT_MAX) idx--; else idx = Modules.GetItemsInContainer()-1; break;
      case Random :
       idx = random(Modules.GetItemsInContainer()); break;
      case Repeat :
       break;
     }

    if ((clp == Clip) || (clp == StopAfter))
     {
      if (idx < 0)
       {
        idx = 0;
        if (clp == StopAfter) play = 0;
       }
      if (idx > Modules.GetItemsInContainer()-1)
       {
        idx = Modules.GetItemsInContainer()-1;
        if (clp == StopAfter) play = 0;
       }
     }
    else if (clp == TurnAround)
     {
      if (idx < 0) idx = idx = Modules.GetItemsInContainer()-1;
      if (idx > Modules.GetItemsInContainer()-1) idx = 0;
     }

    if (idx == -1)
     ActiveDMM = 0;
    else
     ActiveDMM = Modules[idx];

    if (ActiveDMM != olddmm) Load();
    if (play) Play();
   }
  else
   ActiveDMM = 0;
 }

void lpMIDASPlayer::SetPlayOrder(lpPlayOrder po)
 {
  PlayOrder = po;
 }

void lpMIDASPlayer::SetClipping(lpClipping clp)
 {
  Clipping = clp;
 }

lpMIDASPlayer::lpPlayOrder lpMIDASPlayer::GetPlayOrder()
 {
  return PlayOrder;
 }

lpMIDASPlayer::lpClipping lpMIDASPlayer::GetClipping()
 {
  return Clipping;
 }

lpMIDASPlayer::lpState lpMIDASPlayer::GetState()
 {
  return State;
 }

int lpMIDASPlayer::GetTrackActualPos()
 {
  if (ActiveDMM)
   return SongState.position;
  else
   return 0;
 }

void lpMIDASPlayer::SetTrackActualPos(int pos)
 {
  if (pos < 0) pos = 0;
  if (pos > GetTrackLength()-1) pos = GetTrackLength()-1;

  if (GetState(Playing))
   MIDASsetPosition(hPlay, pos);
 }

int lpMIDASPlayer::GetTrackActualPattern()
 {
  if (ActiveDMM)
   return SongState.pattern;
  else
   return 0;
 }

int lpMIDASPlayer::GetTrackActualRow()
 {
  if (ActiveDMM)
   return SongState.row;
  else
   return 0;
 }

int lpMIDASPlayer::GetTrackLength()
 {
  if (ActiveDMM)
   return SongInfo.songLength;
  else
   return 0;
 }

int lpMIDASPlayer::GetTrackPatternCount()
 {
  if (ActiveDMM)
   return SongInfo.numPatterns;
  else
   return 0;
 }

int lpMIDASPlayer::GetTrackInstrumentCount()
 {
  if (ActiveDMM)
   return SongInfo.numInstruments;
  else
   return 0;
 }

int lpMIDASPlayer::GetTrackChannelCount()
 {
  if (ActiveDMM)
   return SongInfo.numChannels;
  else
   return 0;
 }

string lpMIDASPlayer::GetTrackInstrumentName(int i)
 {
  if (ActiveDMM)
   {
    MIDASinstrumentInfo ii;
    MIDASgetInstrumentInfo(hModule, i, &ii);
    return ii.instName;
   }
  else
   return "";
 }
string lpMIDASPlayer::GetTrackName()
 {
  if (ActiveDMM)
   return ActiveDMM->GetTrackName();
  else
   return "";
 }

string lpMIDASPlayer::GetTrackFile()
 {
  if (ActiveDMM)
   return ActiveDMM->GetFileName();
  else
   return "";
 }

lpDMM* lpMIDASPlayer::GetTrackDMM()
 {
  return ActiveDMM;
 }

int lpMIDASPlayer::GetTrackIndex()
 {
  return Modules.Find(ActiveDMM);
 }

const lpMIDASPlayer::lpDMMList* lpMIDASPlayer::GetDMMList()
 {
  return &Modules;
 }

void lpMIDASPlayer::UpdateSongInfo()
 {
  if (GetState(Playing))
   {
    MIDASgetPlayStatus(hPlay, &SongState);
    if (SongState.songLoopCount != 0)
     {
      Stop();
      PlayNextModule();
     }
   }
 }

bool lpMIDASPlayer::TrackChanged()
 {
  bool tc = TrackChange;
  TrackChange = 0;
  return tc;
 }

bool lpMIDASPlayer::GetState(lpState s)
 {
  return (State & s) == s;
 }

void lpMIDASPlayer::SetState(bool en, lpState s)
 {
  if (en)
   State = (lpState) (State | s);
  else
   State = (lpState) (State & ~s);
 }

