#include <cstring.h>
#include <classlib\arrays.h>
#include <midasdll.h>

#define NO_TRACK_INFO "No track info available"
#define NO_FILE_SPEC "{nofilespec}"
#define POLLRATE 50

class lpDMM
 {
  public:
   lpDMM(const lpDMM& dmm)
    {
     fn = dmm.fn;
     track = dmm.track;
    }

   lpDMM(const string& FN, const string& TRACK = NO_TRACK_INFO)
    {
     fn = FN;
     track = TRACK;
    }

   virtual const string& GetTrackName() const
    {
     return track;
    }

   virtual const string& SetTrackName(const string &t)
    {
     return (track = t);
    }

   virtual const string& GetFileName() const
    {
     return fn;
    }

   friend bool operator == (const lpDMM& dmm1, const lpDMM& dmm2)
  		{
     return dmm1.fn == dmm2.fn;
    }

   friend bool operator < (const lpDMM& dmm1, const lpDMM& dmm2)
  		{
     return dmm1.fn < dmm2.fn;
    }

   friend bool operator > (const lpDMM& dmm1, const lpDMM& dmm2)
  		{
     return dmm1.fn > dmm2.fn;
    }

   void RetrieveSongInfo();
  protected:
   string fn;
   string track;
 };

class lpMIDASPlayer
 {
  public:
   typedef TIArrayAsVector<lpDMM> lpDMMList;
   typedef TIArrayAsVectorIterator<lpDMM> lpDMMListIterator;

  protected:
   lpDMMList Modules;
   string ListFN;
   lpDMM *ActiveDMM;
   int iVolume;

  public:
   enum lpPlayOrder {SequentialForward, SequentialBackward, Random, Repeat};
   enum lpClipping {TurnAround, Clip, StopAfter};
   enum lpState {Nothing = 0, Loaded = 1, Playing = 2, Paused = 4};

  protected:
   lpPlayOrder PlayOrder;
   lpClipping Clipping;
   lpState State;

   MIDASmoduleInfo SongInfo;
   MIDASplayStatus SongState;
   MIDASmodule hModule;
   MIDASmodulePlayHandle hPlay;
   bool TrackChange;

  public:
   lpMIDASPlayer();
   ~lpMIDASPlayer();

   void midas_Init();
   void midas_Done();
   void FreePlayList();
   void ReadPlayList(const string& = NO_FILE_SPEC);
   void WritePlayList();
   void SetPlayListName(const string&);
   string GetPlayListName();

   lpDMM* AddModule(const string& fn, const string& track = NO_TRACK_INFO);

   void DeleteIndex(int idx);
   void DeleteTrack(const string &track);
   void DeleteFile(const string &fn);
   void DeleteDMM(lpDMM *dmm);

   void Load();
   void Unload();

   void PlayIndex(int idx);
   void PlayFile(const string &fn);
   void PlayTrack(const string &track);
   void PlayDMM(lpDMM *dmm);

   void SetVolume(int vol);
   void SetVolume()
    {
     SetVolume(iVolume);
    }
   void Play(int pos = 0);
   void Stop();
   void Pause();
   void Resume();

   void PlayNextModule(lpPlayOrder po, lpClipping clp);
   void PlayNextModule(){PlayNextModule(PlayOrder, Clipping);}
   void PlayNextModule(lpPlayOrder po) {PlayNextModule(po, Clipping);}
   void SetPlayOrder(lpPlayOrder po);
   void SetClipping(lpClipping clp);

   lpPlayOrder GetPlayOrder();
   lpClipping GetClipping();
   lpState GetState();

   int GetTrackActualPos();
   void SetTrackActualPos(int pos);
   int GetTrackActualPattern();
   int GetTrackActualRow();

   int GetTrackLength();
   int GetTrackPatternCount();
   int GetTrackInstrumentCount();
   int GetTrackChannelCount();
   string GetTrackInstrumentName(int i);

   string GetTrackName();
   string GetTrackFile();
   lpDMM* GetTrackDMM();
   int GetTrackIndex();

   lpDMM* FindTrackDMM(const string& track);
   lpDMM* FindFileDMM(const string& fn);
   lpDMM* FindIndexDMM(int idx);

   const lpDMMList* GetDMMList();

   void UpdateSongInfo();
   bool TrackChanged();

   bool GetState(lpState s);

  protected:
   void SetState(bool en, lpState);
 };
